/* BBStree.h
**
** `Balanced `Binary `Search `trees
**     Funktionen zur Verwaltung eines binren Baumes:
**
**     int BBStree_empty (BBStree_t tree)
**         - testen, ob der Baum leer ist
**
**     void BBStree_clear (BBStree_t *tree, void (*elfree) (void *elem))
**         - leeren des Baumes
**
**     void **BBStree_insert (BBStree_t *tree, void *elem,
**                            int (*compare) (void *left, void *right))
**         - einfgen eines neuen Knotens in den Baum; `compare' ist die
**           fr Vergleiche innerhalb der Baum-Struktur bentigte Funktion.
**           Das Ergebnis ist:
**              NULL
**                  fr erfolgreiches Einfgen,
**              ein Zeiger auf einen Zeiger auf das Datenelement
**                  falls schon ein entsprechendes Element vorhanden war
**
**     void **BBStree_get (BBStree_t tree, void *key,
**                      int (*compare) (void *key, void *elem))
**         - lesen eines Knotens aus dem Baumes. Hierbei ist `compare'
**           eine Funktion fr Vergleiche eines Schlssels mit den Elementen
**           des Baum.
**           Das Ergebnis ist:
**              NULL
**                  falls ein Knoten mit dem gegebenen Schlssel nicht
**                  existiert
**              ein Zeiger auf die Speicherstelle in der Baumstruktur, die den
**              Zeiger auf die Daten enthlt
**                  in jedem anderen Fall.
**
**     int BBStree_lotraverse (BBStree_t tree,
**                             int (*eval) (void *node_data, void *static_data),
**                             void *static_data)
**         - Traversieren des gesamten Baumes (pre-order traversal)
**
**     int BBStree_rotraverse (BBStree_t tree,
**                             int (*eval) (void *node_data, void *static_data),
**                             void *static_data)
**         - Traversieren des gesamten Baumes (post-order traversal)
**
**     int BBStree_iotraverse (BBStree_t tree,
**                             int (*eval) (void *node_data, void *static_data),
**                             void *static_data)
**         - Traversieren des gesamten Baumes (in-order traversal)
**
**     int BBStree_delete (BBStree_t tree, void *key,
**                         int (*compare) (void *key, void *elem),
**                         void (*elfree) (void *elem));
**
**     Diese Baum-Struktur soll nur ber diese drei Funktionen von auen
**     ansprechbar sein.
*/
#ifndef _BBSTREE_H
#define _BBSTREE_H

/* Die fr die jeweiligen Vergleiche bentigten Funktionstypen.
*/
typedef int (*compare_func) (void *left, void *right);
typedef int (*keycomp_func) (void *key, void *elem);
typedef void (*release_func) (void *elem);
typedef int (*trav_func) (void *node_data, void *static_data);

struct bbs_node;

typedef struct bbs_node *BBStree_t;

/* Test daraufhin, ob der Baum leer ist  */
extern int BBStree_empty (BBStree_t tree);

/* BBStree leeren */
extern void BBStree_clear (BBStree_t *tree, release_func elfree);

/* Element in den BBStree einfgen */
extern void **BBStree_insert (BBStree_t *tree, void *elem, compare_func elcomp);

/* Element aus einem BBStree lschen */
extern int BBStree_delete (BBStree_t *tree, void *key, keycomp_func keycomp,
			   release_func elfree);

/* Element im BBStree suchen */
extern void **BBStree_get (BBStree_t tree, void *key, keycomp_func keycmp);

/* Traversierungsfunktionen */

/* pre-order traversel (lo) */
extern int BBStree_lotraverse (BBStree_t tree, trav_func eval,
			       void *static_data);

/* post-order traversel (ro) */
extern int BBStree_rotraverse (BBStree_t tree, trav_func eval,
			       void *static_data);

/* in-order traversel (io) */
extern int BBStree_iotraverse (BBStree_t tree, trav_func eval,
			       void *static_data);

#endif /* defined (_BBSTREE_H) */
