/* rec.c
**
** Database Interface for the user/admin databases
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

#include "prefs.h"
#include "ifconf.h"
#include "dbif_priv.h"
#include "calc_fsize.h"

/* Erzeuge einen Benutzerdatensatz aus den Daten fr Pawort, Foldername und
** Typ und - im Fall, da ein Datensatz fr eine Domain erzeugt werden soll -
** Zahl der Benutzer innerhalb der Domain und maximale Anzahl an zugelassenen
** Benutzern fr diese Domain.
** Pawort und Foldername werden mit derLnge als zustzlichem Parameter
** angegeben.
*/
dbif_rec *dbif_mkrecl (char *passwd, size_t pwl,
		       char *folder, size_t fll,
		       size_t msgmax, off_t fldmax,
		       int rtype,
		       ...)
{
/* Neue Idee!
** Wenn ich hier die Argumentliste umorganisiere und einzelnen Folder als
** variable Anzahl von Argumenten hinter `rtype' unterbringe, dann kann ich
** mit LLTO mehrere Zieladressen mitfhren.
*/
    int nusers = 0, maxusers = 0;
    dbif_rec *res;
    size_t ressz = sizeof (dbif_rec) + pwl + fll + 5;
    va_list ap;
    char *p;
    if (rtype == RT_DOMAIN || rtype == RT_NSDOMAIN) {
	va_start (ap, rtype),
	nusers = va_arg (ap, int);
	maxusers = va_arg (ap, int);
	va_end (ap);
	ressz += 2 * sizeof (int);
    }
    if (!(res = malloc (ressz))) {
	if_errno = DBIF_ENOMEM;
	return res;
    }
    p = (char *) res + sizeof (dbif_rec);
    res->passwd = p; memcpy (p, passwd, pwl); p[pwl] = '\0';
    p += pwl + 1;
    res->folder = p; memcpy (p, folder, fll); p[fll] = '\0';
    p += fll + 1;
    res->flags = p;
    *p++ = (char) rtype;
    res->uid = prefs.vxuid; res->gid = prefs.vxgid;
    res->msg_limit = msgmax; res->fld_limit = fldmax;
    if (rtype == RT_DOMAIN || rtype == RT_NSDOMAIN) {
	set_int (nusers, p);
	set_int (maxusers, p + sizeof(int));
    }
    if_errno = DBIF_NOERR;
    return res;
}

/* Konvertiere einen Datensatz aus der Datenbank in einen Benutzerdatensatz */
dbif_rec *dbif_mkreca (char *p)
{
    size_t pwl, fll;
    char *passwd, *folder;
    int rtype, nusers = 0, maxusers = 0;
    size_t msgmax; off_t fldmax;
    pwl = get_int(p); p += sizeof(int);
    fll = get_int(p); p += sizeof(int);
    passwd = p; p += pwl;
    folder = p; p += fll;
    msgmax = get_size (p); p += sizeof(size_t);
    fldmax = get_offs (p); p += sizeof(off_t);
    rtype = (int) *p++;
    if (rtype == RT_DOMAIN || rtype == RT_NSDOMAIN) {
	nusers = get_int (p); p += sizeof(int);
	maxusers = get_int (p); p += sizeof(int);
    }
    return dbif_mkrecl (passwd, pwl, folder, fll,
			msgmax, fldmax, rtype, nusers, maxusers);
}

/* Erzeuge einen Benutzerdatensatz aus den Daten fr Pawort, Foldername und
** Flags; die Flags werden als NULL-terminierte Liste von Flagnamen (Zeichen-
** ketten hinter dem Foldernamen bergeben.
*/
dbif_rec *dbif_mkrec (char *passwd, char *folder,
		      size_t msgmax, off_t fldmax,
		      int rtype, ...)
{
/* Neue Idee!
** Wenn ich hier die Argumentliste umorganisiere und einzelnen Folder als
** variable Anzahl von Argumenten hinter `rtype' unterbringe, dann kann ich
** mit LLTO mehrere Zieladressen mitfhren.
*/
    va_list ap;
    int nusers, maxusers;
    if (rtype == RT_DOMAIN || rtype == RT_NSDOMAIN) {
	va_start (ap, rtype);
	nusers = va_arg (ap, int);
	maxusers = va_arg (ap, int);
	va_end (ap);
	return dbif_mkrecl (passwd, strlen (passwd),
			    folder, strlen (folder),
			    msgmax, fldmax,
			    rtype,
			    nusers, maxusers);
    }
    /* NEW: 08/23/2000
    ** i inserted a new header file containing an inline function which
    ** calculates the size of the folder data (depending on the record type)
    ** for me.
    */
    return dbif_mkrecl (passwd, strlen (passwd),
			folder, calc_fsize (rtype, folder),
			msgmax, fldmax,
			rtype);
}

/* dbif_rectype (userrec)
**     returns the type of the user record `userrec'
*/
int dbif_rectype (const dbif_rec *r)
{
    return (int) *(r->flags);
}

/* dbif_setlim (domrec, users, maxusers)
**     sets the limits of a domain record; if the record is not a domain
**     record, it returns a non-zero value.
*/
int dbif_setlim (dbif_rec *r, int nusers, int maxusers)
{
    char *p = r->flags;
    if (((int) *p & 0xff) == RT_DOMAIN || ((int) *p & 0xff) == RT_NSDOMAIN) {
	++p;               set_int (nusers, p);
	p += sizeof (int); set_int (maxusers, p);
	return 0;
    }
    return 1;
}

/* dbif_getlim (domrec, &users, &maxusers)
**     reads the limits of a domain record into the var arguments `users' and
**     `maxusers'; if the record is not a domain record, it returns a non-zero
**     value.
*/
int dbif_getlim (dbif_rec *r, int *nusers, int *maxusers)
{
    char *p = r->flags;
    if (((int) *p & 0xff) == RT_DOMAIN || ((int) *p & 0xff) == RT_NSDOMAIN) {
	++p;               *nusers = get_int (p);
	p += sizeof (int); *maxusers = get_int (p);
	return 0;
    }
    return 1;
}

/* dbif_free (userrec)
**    releases the memory which was allocated for `userrec'.
*/
void dbif_free (dbif_rec *p)
{
    free (p);
}
